<?php
if ( ! class_exists( 'IPTrank_AdminTable' )) {

    /**
     * Class for display table of smart pop up
     */
    class IPTrank_AdminTable extends WP_List_Table
    {
        /**
         * Get a list of columns. The format is:
         * 'internal-name' => 'Title'
         *
         * @return array
         */
        public function get_columns() 
        {
            $posts_columns                 = array();
            $posts_columns['cb']           = '<input type="checkbox" />';
            $posts_columns['keyword']      = __( 'Keyword', 'iptrank' );
            $posts_columns['site']         = __( 'Site', 'iptrank' );
            $posts_columns['rank']         = __( 'Rank', 'iptrank' );
            $posts_columns['last_update']  = __( 'Last Update', 'iptrank' );
            $posts_columns['type']         = __( 'Type', 'iptrank' );
            $posts_columns['group']        = __( 'Group', 'iptrank' );
            $posts_columns['actions']      = __( 'Actions', 'iptrank' );
            return $posts_columns;
        }


        /**
         * Get a list of sortable columns. The format is:
         * 'internal-name' => 'orderby'
         * or
         * 'internal-name' => array( 'orderby', true )
         *
         * The second format will make the initial sorting order be descending
         *
         * @return array
         */
        public function get_sortable_columns() 
        {
            return array(
                'keyword'       => 'keyword',
                'site'          => 'site',
                'rank'          => 'rank',
                'type'          => 'type',
                'last_update'   => 'lastupdate',
                'group'         => 'group'
            );
        }

        /**
         * Get a list of all, hidden and sortable columns, with filter applied
         *
         * @return array
         */
        public function get_column_info() 
        {
            $columns  = $this->get_columns();
            $hidden   = array();
            $sortable = array();
            $this->_column_headers = array( $columns, $hidden, $sortable );
            return $this->_column_headers;
        }

        /**
         * Prepares the list of items for displaying.
         * @uses WP_List_Table::set_pagination_args()
         *
         */
        public function prepare_items() 
        {
            global $wp_query, $per_page;
            
            $page = 1;
            if ( ! empty( $_GET['paged'] )) {
                $page = absint( $_GET['paged'] );
            }
            $per_page = $this->get_items_per_page( 'edit_iptrank-list_per_page' );
            $per_page = apply_filters( 'edit_posts_per_page', $per_page, 'iptrank-list' );
            
            $group   = 0;
            $site    = '';
            $keyword = '';
            if ( ! empty( $_GET['group'] )) {
                $group = absint( $_GET['group'] );
            }
            if ( ! empty( $_GET['site'] )) {
                $site = esc_url_raw( urldecode( $_GET['site'] ) );
            }
            if ( ! empty( $_GET['keyword'] )) {
                $keyword = apply_filters( 'single_post_title', $_GET['keyword'] );
            }
            
            $model = new IPTrank_Model();
            $this->items = $model->get_ranks_search( $site, $group, $keyword, $per_page, $page );

            $total_items = $model->get_found_item();
            $total_pages = ceil( $total_items / $per_page );
            $this->set_pagination_args( array(
                'total_items' => $total_items,
                'total_pages' => $total_pages,
                'per_page' => $per_page
            ) );
        }

        /**
         * Get column id
         *
         * @param object $item
         * @return void
         */
        public function column_cb( $item ) 
        {
            return '<input type="checkbox" value="' . $item->rank_id . '" name="rank[]" id="cb-select-' . $item->rank_id . '">';
        }

        /**
         * Get column keyword
         *
         * @param object $item
         * @return void
         */
        public function column_keyword( $item ) 
        {
            $link  = admin_url() . 'admin.php?page=iptrank&action=view_keyword&id='.$item->rank_id;
            $kword = $item->rank_keyword;
            return $kword;
        }

        /**
         * Get column rank
         *
         * @param object $item
         * @return void
         */
        public function column_rank( $item ) 
        {
            $rank = $item->rank_value;
            if ( $rank > 100 ) {
                $rank = '100+';
            }
            $rank = '<span class="iptrank-rank-value">' . $rank . '</span>';
            //Progress
            if ( $item->rank_progress > 0 ) {
                $icon = '<span class="ipadm-btn ipadm-btn-success ipadm-btn-small iptrank-rank-status"><i class="faip fa-long-arrow-up"></i></span>';
            } elseif ( $item->rank_progress == 0 ) {
                $icon = '<span class="ipadm-btn ipadm-btn-warning ipadm-btn-small iptrank-rank-status"><i class="faip fa-arrows-h"></i></span>';
            } else {
                $icon = '<span class="ipadm-btn ipadm-btn-danger ipadm-btn-small iptrank-rank-status"><i class="faip fa-long-arrow-down"></i></span>';
            }
            $rank .= $icon;
            if ( ! empty( $item->rank_progress )) {
                if ( $item->rank_progress > 0 ) {
                    $val = '+'.$item->rank_progress;
                } else {
                    $val = $item->rank_progress;
                }
                $rank .= "( $val )";
            }
            return $rank;
        }

        /**
         * Get column site
         *
         * @param object $item
         * @return void
         */
        public function column_site( $item ) 
        {
            $site = $item->rank_site;
            return $site;
        }

        /**
         * Get column last update
         *
         * @param object $item
         * @return void
         */
        public function column_last_update( $item ) 
        {
            $date = '<div class="iptrank-update-status">' .
                        '<span class="iptrank-last">' . mysql2date( 'd M y, h:i', $item->rank_modified ) . '</span>' .
                        '<span class="iptrank-spinner" style="display:none;"></span>' .
                    '</div>';
            return $date;
        }

        /**
         * Get column type
         *
         * @param object $item
         * @return void
         */
        public function column_type( $item ) 
        {
            $type = __( 'Manual', 'iptrank' );
            if ( $item->rank_type == 2 ) {
                $type = __( 'Auto', 'iptrank' );
            }
            return $type;
        }

        /**
         * Get column group
         *
         * @param object $item
         * @return void
         */
        public function column_group( $item ) 
        {
            $gname = '-';
            if ( ! empty( $item->group_id )) {
                $model = new IPTrank_Model();
                if ( $group = $model->get_group( $item->group_id ) ) {
                    $gname = $group->group_name;
                }
            }
            return $gname;
        }

        /**
         * Get column actions
         *
         * @param object $item
         * @return void
         */
        public function column_actions( $item ) 
        {
            $linkUpd  = '#toUpdate_'.$item->rank_id;
            $linkView = '#toView_'.$item->rank_id;
            $linkDel  = '#toDelete_'.$item->rank_id;
            $actions  = '<div class="iptrank-actions">
                            <a class="iptrank-keyword-update ipadm-btn ipadm-btn-primary" href="' . $linkUpd . '" title="Update keyword ranking">' . 
                                '<i class="faip fa-refresh"></i>
                            </a>
                            <a class="iptrank-keyword-view ipadm-btn ipadm-btn-success" href="' . $linkView . '" title="View keyword report">' . 
                                '<i class="faip fa-eye"></i>
                            </a>
                            <a class="iptrank-keyword-delete ipadm-btn ipadm-btn-danger" href="' . $linkDel . '">' .
                                '<i class="faip fa-trash-o"></i>
                            </a>
                        </div>';
            
            return $actions;
        }

        /**
         * Get column default
         *
         */
        public function column_default() 
        {
            return '';
        }
        
        /**
         * Displaying table nav
         */
        function display_tablenav( $which ) 
        {
            $model = new IPTrank_Model();
            
            if ( $which == 'bottom' ) {
                parent::display_tablenav( $which );
            } else {
            
                $cgroup  = 0;
                $csite   = '';
                $keyword = '';
                if ( ! empty( $_GET['group'] )) {
                    $cgroup = absint( $_GET['group'] );
                }
                if ( ! empty( $_GET['site'] )) {
                    $csite = esc_url_raw( urldecode( $_GET['site'] ) );
                }
                if ( ! empty( $_GET['keyword'] )) {
                    $keyword = apply_filters( 'single_post_title', $_GET['keyword'] );
                }
            ?>
                <div class="tablenav top">
                    <div class="alignleft actions bulkactions">
                        <select name="action" id="iptrank-action-name">
                            <option selected="selected" value="-1"><?php _e( 'Bulk Actions' ); ?></option>
                            <option value="update"><?php _e( 'Update' ); ?></option>
                            <option value="delete"><?php _e( 'Delete' ); ?></option>
                        </select>
                        <input type="submit" value="<?php _e( 'Apply' ); ?>" class="button action" id="iptrank-doaction" name="">
                    </div>
                    
                    <div class="alignleft actions">
                        <select name="site">
                            <option value="0" selected="selected"><?php _e( 'All Sites', 'iptrank' ); ?></option>
                            <?php
                            $sites = $model->get_sites();
                            foreach( $sites as $site )
                            {
                                $sltd = '';
                                if ( $csite == $site ) {
                                    $sltd = ' selected="selected"';
                                }
                                echo '<option value="' . urlencode( $site ) . '"' . $sltd . '>' . $site . '</option>';
                            }
                            ?>
                        </select>
                        <select name="group">
                            <option value="0"><?php _e( 'All Groups', 'iptrank' ); ?></option>
                            <?php
                            $groups = $model->get_groups();
                            foreach( $groups as $group )
                            {
                                $sltd = '';
                                if ( $group->group_id == $cgroup ) {
                                    $sltd = ' selected="selected"';
                                }
                                echo '<option value="' . $group->group_id . '"' . $sltd . '>' . $group->group_name . '</option>';
                            }
                            ?>
                        </select>
                        <input type="submit" value="<?php _e( 'Filter' ); ?>" class="button" name="">
                    </div>
                    
                    <div class="alignright">
                        <input type="text" name="keyword" id="iptrank-input-keyword" placeholder="<?php _e( 'Keyword...', 'iptrank' ); ?>" value="<?php echo $keyword; ?>">
                        <input type="submit" value="<?php _e( 'Search' ); ?>" class="button" name="">
                    </div>
                </div>
                
            <?php
            }
        }
    }

}
?>