<?php
if ( ! class_exists( 'IPTrank_Model' )) {

    /**
     * Class model
     */
    class IPTrank_Model
    {
        /**
         * Get ranks
         *
         * @param int $perpage
         * @param int $page
         * @return array
         */
        public function get_ranks( $perpage = 0, $page = 1 )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank';
            $query = "
                SELECT SQL_CALC_FOUND_ROWS DISTINCT * 
                    FROM $table 
                    ORDER BY rank_value
            ";
            if ( $perpage != 0 ) {
                $offset = $perpage * ($page-1);
                $query .= " LIMIT $perpage OFFSET $offset";
            }
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get ranks
         *
         * @param string $site
         * @param int    $group
         * @param int    $perpage
         * @param int    $page
         * @return array
         */
        public function get_ranks_search( $site = '', $group = 0, $keyword = '', $perpage = 0, $page = 1 )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank';
            $query = "
                SELECT SQL_CALC_FOUND_ROWS DISTINCT * 
                    FROM $table
                    WHERE 1=1";
            
            if ( ! empty( $site )) {
                $query .= " AND rank_site = '$site'";
            }
            if ( ! empty( $group )) {
                $query .= " AND group_id = '$group'";
            }
            if ( ! empty( $keyword )) {
                $query .= " AND rank_keyword LIKE '%$keyword%'";
            }
            $query .= " ORDER BY rank_value";
            if ( $perpage != 0 ) {
                $offset = $perpage * ($page-1);
                $query .= " LIMIT $perpage OFFSET $offset";
            }
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get ranks
         *
         * @param array $ids
         * @return array
         */
        public function get_ranks_byid( $ids )
        {
            global $wpdb;
            
            $ids   = (array)$ids;
            $table = $wpdb->prefix . 'iptrank_rank';
            $query = "
                SELECT * 
                    FROM $table 
                    WHERE rank_id IN ( " . implode( ',', $ids ) . " )
                    ORDER BY rank_value
            ";
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get rank
         *
         * @param  int $id
         * @return array
         */
        public function get_rank( $id )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank';
            $sql   = "
                SELECT * 
                    FROM $table 
                    WHERE rank_id = %d
            ";
            $id      = absint( $id );
            $query   = $wpdb->prepare( $sql, $id );
            $results = $wpdb->get_row( $query );
            return $results;
        }
        
        /**
         * Get rank by key
         *
         * @param  string $keyword
         * @param  string $site
         * @return array
         */
        public function get_rank_bykey( $keyword, $site )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank';
            $sql   = "
                SELECT * 
                    FROM $table 
                    WHERE rank_keyword = '$keyword' 
                    AND rank_site = '$site'
            ";
            $results = $wpdb->get_row( $sql );
            return $results;
        }
        
        /**
         * Get ranks in a group
         *
         * @param int $groupId
         * @param int $perpage
         * @param int $page
         * @return array
         */
        public function get_group_ranks( $groupId, $perpage = 0, $page = 1 )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank';
            $sql   = "
                SELECT * 
                    FROM $table 
                    WHERE group_id = %d
                    ORDER BY rank_value
            ";
            if ( $perpage != 0 ) {
                $offset = $perpage * ($page-1);
                $sql   .= " LIMIT $perpage OFFSET $offset";
            }
            $id      = absint( $groupId );
            $query   = $wpdb->prepare( $sql, $id );
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get site statistic data
         *
         * @return array
         */
        public function get_site_stat_data( $site, $start = null, $end = null )
        {
            global $wpdb;
            
            $page_id = absint( $page_id );
            $table1  = $wpdb->prefix . 'iptrank_rank';
            $table2  = $wpdb->prefix . 'iptrank_rank_record';
            $query   = "
                SELECT *,
                       DATE_FORMAT( $table2.reco_date, '%Y-%m-%d') AS rdate
                    FROM $table1
                    JOIN $table2 ON $table1.rank_id = $table2.rank_id
                    WHERE $table1.rank_site = '$site'";
            
            if ( ! empty( $start ) && ! empty( $end )) 
            {
                if ( $start == $end ) {
                    $query .= " AND DATE_FORMAT( $table2.reco_date, '%Y-%m-%d' ) = '$start'";
                } else {
                    $query .= " AND $table2.reco_date BETWEEN '$start' AND '$end'";
                }
            }
            elseif ( ! empty( $start )) {
                $query .= " AND DATE_FORMAT( $table2.reco_date, '%Y-%m-%d' ) = '$start'";
            }
            elseif ( ! empty( $end )) {
                $query .= " AND DATE_FORMAT( $table2.reco_date, '%Y-%m-%d' ) = '$end'";
            }
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get keyword statistic data
         *
         * @param  int    $id
         * @param  string $start
         * @param  string $end
         * @return array
         */
        public function get_keyword_stat_data( $id, $start = null, $end = null )
        {
            global $wpdb;
            
            $id      = absint( $id );
            $table1  = $wpdb->prefix . 'iptrank_rank';
            $table2  = $wpdb->prefix . 'iptrank_rank_record';
            $query   = "
                SELECT *,
                       $table2.reco_value AS rank,
                       DATE_FORMAT( $table2.reco_date, '%Y-%m-%d') AS rdate
                    FROM $table1
                    JOIN $table2 ON $table1.rank_id = $table2.rank_id
                    WHERE $table1.rank_id = '$id'";
            
            if ( ! empty( $start ) && ! empty( $end )) 
            {
                if ( $start == $end ) {
                    $query .= " AND DATE_FORMAT( $table2.reco_date, '%Y-%m-%d' ) = '$start'";
                } else {
                    $query .= " AND $table2.reco_date BETWEEN '$start' AND '$end'";
                }
            }
            elseif ( ! empty( $start )) {
                $query .= " AND DATE_FORMAT( $table2.reco_date, '%Y-%m-%d' ) = '$start'";
            }
            elseif ( ! empty( $end )) {
                $query .= " AND DATE_FORMAT( $table2.reco_date, '%Y-%m-%d' ) = '$end'";
            }
            $query  .= " GROUP BY rdate";
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get keyword statistic data
         *
         * @param  string $site
         * @param  int    $group
         * @param  int    $date_method
         * @param  int    $year
         * @param  int    $month
         * @return array
         */
        public function get_report_main_data( $site = '', $group = 0, $date_method = 1, $year = 0, $month = 0 )
        {
            global $wpdb;
            
            $id     = absint( $id );
            $table1 = $wpdb->prefix . 'iptrank_rank';
            $table2 = $wpdb->prefix . 'iptrank_rank_record';
            $query  = "
                SELECT SUM($table2.reco_value) AS rank,";
            
            if ( $date_method == 1 ) {
                $query .= " DATE_FORMAT( $table2.reco_date, '%d') AS rday,";
                $query .= " DATE_FORMAT( $table2.reco_date, '%Y-%m-%d') AS rdate";
            } else {
                $query .= " DATE_FORMAT( $table2.reco_date, '%d') AS rday,";
                $query .= " DATE_FORMAT( $table2.reco_date, '%m') AS rmonth,";
                $query .= " DATE_FORMAT( $table2.reco_date, '%Y-%m') AS rdate";
            }
            $query .= " FROM $table1
                        JOIN $table2 ON $table1.rank_id = $table2.rank_id WHERE 1=1";
            
            if ( ! empty( $site )) {
                $query .= " AND $table1.rank_site = '$site'";
            }
            if ( ! empty( $group )) {
                $query .= " AND $table1.group_id = '$group'";
            }
            if ( $date_method == 1 && ! empty( $year ) && ! empty( $month )) {
                $query .= " AND DATE_FORMAT( $table2.reco_date, '%Y' ) = '$year'";
                $query .= " AND DATE_FORMAT( $table2.reco_date, '%m' ) = '$month'";
            } else if ( $date_method == 2 && ! empty( $year )) {
                $query .= " AND DATE_FORMAT( $table2.reco_date, '%Y' ) = '$year'";
            }
            $query  .= " GROUP BY rdate";
            
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get outranking up
         *
         * @param  string $site
         * @param  int    $group
         * @param  int    $date_method
         * @param  int    $year
         * @param  int    $month
         * @return array
         */
        public function get_outranking_up( $site = '', $group = 0, $date_method = 1, $year = 0, $month = 0 )
        {
            global $wpdb;
            
            $table  = $wpdb->prefix . 'iptrank_rank';
            $query  = "
                SELECT COUNT(rank_id) AS count 
                FROM $table 
                WHERE rank_progress > 0";
            
            if ( ! empty( $site )) {
                $query .= " AND rank_site = '$site'";
            }
            if ( ! empty( $group )) {
                $query .= " AND group_id = '$group'";
            }
            if ( $date_method == 1 && ! empty( $year ) && ! empty( $month )) {
                $query .= " AND DATE_FORMAT( rank_modified, '%Y' ) = '$year'";
                $query .= " AND DATE_FORMAT( rank_modified, '%m' ) = '$month'";
            } else if ( $date_method == 2 && ! empty( $year )) {
                $query .= " AND DATE_FORMAT( rank_modified, '%Y' ) = '$year'";
            }
            
            $result = $wpdb->get_var( $query );
            if ( empty( $result )) {
                $result = 0;
            }
            return $result;
        }
        
        /**
         * Get outranking down
         *
         * @param  string $site
         * @param  int    $group
         * @param  int    $date_method
         * @param  int    $year
         * @param  int    $month
         * @return array
         */
        public function get_outranking_down( $site = '', $group = 0, $date_method = 1, $year = 0, $month = 0 )
        {
            global $wpdb;
            
            $table  = $wpdb->prefix . 'iptrank_rank';
            $query  = "
                SELECT COUNT(rank_id) AS count 
                FROM $table 
                WHERE rank_progress < 0";
            
            if ( ! empty( $site )) {
                $query .= " AND rank_site = '$site'";
            }
            if ( ! empty( $group )) {
                $query .= " AND group_id = '$group'";
            }
            if ( $date_method == 1 && ! empty( $year ) && ! empty( $month )) {
                $query .= " AND DATE_FORMAT( rank_modified, '%Y' ) = '$year'";
                $query .= " AND DATE_FORMAT( rank_modified, '%m' ) = '$month'";
            } else if ( $date_method == 2 && ! empty( $year )) {
                $query .= " AND DATE_FORMAT( rank_modified, '%Y' ) = '$year'";
            }
            
            $result = $wpdb->get_var( $query );
            if ( empty( $result )) {
                $result = 0;
            }
            return $result;
        }
        
        /**
         * Get ranking top
         *
         * @param  int    $top
         * @param  string $site
         * @param  int    $group
         * @param  int    $date_method
         * @param  int    $year
         * @param  int    $month
         * @return array
         */
        public function get_ranking_top( $top, $site = '', $group = 0, $date_method = 1, $year = 0, $month = 0 )
        {
            global $wpdb;
            
            $top    = absint( $top );
            $table  = $wpdb->prefix . 'iptrank_rank';
            $query  = "
                SELECT COUNT(rank_id) AS count 
                FROM $table 
                WHERE rank_value <= $top";
            
            if ( ! empty( $site )) {
                $query .= " AND rank_site = '$site'";
            }
            if ( ! empty( $group )) {
                $query .= " AND group_id = '$group'";
            }
            if ( $date_method == 1 && ! empty( $year ) && ! empty( $month )) {
                $query .= " AND DATE_FORMAT( rank_modified, '%Y' ) = '$year'";
                $query .= " AND DATE_FORMAT( rank_modified, '%m' ) = '$month'";
            } else if ( $date_method == 2 && ! empty( $year )) {
                $query .= " AND DATE_FORMAT( rank_modified, '%Y' ) = '$year'";
            }
            $result = $wpdb->get_var( $query );
            if ( empty( $result )) {
                $result = 0;
            }
            return $result;
        }
        
        /**
         * Get ranking not top
         *
         * @param  string $site
         * @param  int    $group
         * @param  int    $date_method
         * @param  int    $year
         * @param  int    $month
         * @return array
         */
        public function get_ranking_not_top( $site = '', $group = 0, $date_method = 1, $year = 0, $month = 0 )
        {
            global $wpdb;
            
            $top    = absint( $top );
            $table  = $wpdb->prefix . 'iptrank_rank';
            $query  = "
                SELECT COUNT(rank_id) AS count 
                FROM $table 
                WHERE ( rank_value > 100 OR rank_value = 0 )";
            
            if ( ! empty( $site )) {
                $query .= " AND rank_site = '$site'";
            }
            if ( ! empty( $group )) {
                $query .= " AND group_id = '$group'";
            }
            if ( $date_method == 1 && ! empty( $year ) && ! empty( $month )) {
                $query .= " AND DATE_FORMAT( rank_modified, '%Y' ) = '$year'";
                $query .= " AND DATE_FORMAT( rank_modified, '%m' ) = '$month'";
            } else if ( $date_method == 2 && ! empty( $year )) {
                $query .= " AND DATE_FORMAT( rank_modified, '%Y' ) = '$year'";
            }
            
            $result = $wpdb->get_var( $query );
            if ( empty( $result )) {
                $result = 0;
            }
            return $result;
        }
        
        /**
         * Get rank groups
         *
         * @param int $perpage
         * @param int $page
         * @return array
         */
        public function get_groups( $perpage = 0, $page = 1 )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank_group';
            $query = "
                SELECT SQL_CALC_FOUND_ROWS * 
                    FROM $table 
                    ORDER BY group_name
            ";
            if ( $perpage != 0 ) {
                $offset = $perpage * ($page-1);
                $query .= " LIMIT $perpage OFFSET $offset";
            }
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get rank groups by slug
         *
         * @param string $slug
         * @param int $perpage
         * @param int $page
         * @return array
         */
        public function get_groups_byslug( $slug, $perpage = 0, $page = 1 )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank_group';
            $query = "
                SELECT SQL_CALC_FOUND_ROWS * 
                    FROM $table 
                    WHERE group_slug = '$slug'
                    ORDER BY group_name
            ";
            if ( $perpage != 0 ) {
                $offset = $perpage * ($page-1);
                $query .= " LIMIT $perpage OFFSET $offset";
            }
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get rank group
         *
         * @param int $id
         * @return array
         */
        public function get_group( $id )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank_group';
            $sql   = "
                SELECT * 
                    FROM $table 
                    WHERE group_id = %d
            ";
            $id      = absint( $id );
            $query   = $wpdb->prepare( $sql, $id );
            $results = $wpdb->get_row( $query );
            return $results;
        }
        
        /**
         * Get rank records
         *
         * @param int $rank_id
         * @param int $perpage
         * @param int $page
         * @return array
         */
        public function get_records( $rank_id = 0, $perpage = 0, $page = 1 )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank_record';
            $query = "
                SELECT * 
                    FROM $table
                    WHERE 1=1
            ";
            if ( ! empty( $rank_id )) {
                $query .= " AND rank_id = '$rank_id'";
            }
            $query .= ' ORDER BY reco_date DESC';
            if ( $perpage != 0 ) {
                $offset = $perpage * ($page-1);
                $query .= " LIMIT $perpage OFFSET $offset";
            }
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get best rank records
         *
         * @param int $rank_id
         * @param int $perpage
         * @param int $page
         * @return array
         */
        public function get_best_records( $rank_id = 0, $perpage = 0, $page = 1 )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank_record';
            $query = "
                SELECT * 
                    FROM $table
                    WHERE 1=1
            ";
            if ( ! empty( $rank_id )) {
                $query .= " AND rank_id = '$rank_id'";
            }
            $query .= ' ORDER BY reco_value ASC';
            if ( $perpage != 0 ) {
                $offset = $perpage * ($page-1);
                $query .= " LIMIT $perpage OFFSET $offset";
            }
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get rank records site
         *
         * @param  int    $count
         * @param  string $site
         * @param  int    $group
         * @param  int    $date_method
         * @param  int    $year
         * @param  int    $month
         * @return array
         */
        public function get_last_records_site( $count = 0, $site = '', $group = 0, $date_method = 1, $year = 0, $month = 0 )
        {
            global $wpdb;
            $table1 = $wpdb->prefix . 'iptrank_rank';
            $table2 = $wpdb->prefix . 'iptrank_rank_record';
            $query  = "
                SELECT * 
                    FROM $table1
                    JOIN $table2 ON $table1.rank_id = $table2.rank_id
                    WHERE 1=1
            ";
            if ( ! empty( $site )) {
                $query .= " AND $table1.rank_site = '$site'";
            }
            if ( ! empty( $group )) {
                $query .= " AND $table1.group_id = '$group'";
            }
            if ( $date_method == 1 && ! empty( $year ) && ! empty( $month )) {
                $query .= " AND DATE_FORMAT( $table2.reco_date, '%Y' ) = '$year'";
                $query .= " AND DATE_FORMAT( $table2.reco_date, '%m' ) = '$month'";
            } else if ( $date_method == 2 && ! empty( $year )) {
                $query .= " AND DATE_FORMAT( $table2.reco_date, '%Y' ) = '$year'";
            }
            
            $query .= " ORDER BY $table2.reco_date DESC";
            if ( $count != 0 ) {
                $query .= " LIMIT $count OFFSET 0";
            }
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get rank records site
         *
         * @param  int    $count
         * @param  string $site
         * @param  int    $group
         * @param  int    $date_method
         * @param  int    $year
         * @param  int    $month
         * @return array
         */
        public function get_best_records_site( $count = 0, $site = '', $group = 0, $date_method = 1, $year = 0, $month = 0 )
        {
            global $wpdb;
            $table1 = $wpdb->prefix . 'iptrank_rank';
            $table2 = $wpdb->prefix . 'iptrank_rank_record';
            $query  = "
                SELECT * 
                    FROM $table1
                    JOIN $table2 ON $table1.rank_id = $table2.rank_id
                    WHERE 1=1
            ";
            if ( ! empty( $site )) {
                $query .= " AND $table1.rank_site = '$site'";
            }
            if ( ! empty( $group )) {
                $query .= " AND $table1.group_id = '$group'";
            }
            if ( $date_method == 1 && ! empty( $year ) && ! empty( $month )) {
                $query .= " AND DATE_FORMAT( $table2.reco_date, '%Y' ) = '$year'";
                $query .= " AND DATE_FORMAT( $table2.reco_date, '%m' ) = '$month'";
            } else if ( $date_method == 2 && ! empty( $year )) {
                $query .= " AND DATE_FORMAT( $table2.reco_date, '%Y' ) = '$year'";
            }
            
            $query .= " ORDER BY $table2.reco_value ASC";
            if ( $count != 0 ) {
                $query .= " LIMIT $count OFFSET 0";
            }
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get went up
         *
         * @param  int    $count
         * @param  string $site
         * @param  int    $group
         * @param  int    $date_method
         * @param  int    $year
         * @param  int    $month
         * @return array
         */
        public function get_went_up( $count = 0, $site = '', $group = 0, $date_method = 1, $year = 0, $month = 0 )
        {
            global $wpdb;
            $table1 = $wpdb->prefix . 'iptrank_rank';
            $table2 = $wpdb->prefix . 'iptrank_rank_group';
            $query  = "
                SELECT * 
                    FROM $table1
                    JOIN $table2 ON $table1.group_id = $table2.group_id
                    WHERE 1=1 AND $table1.rank_progress > 0
            ";
            if ( ! empty( $site )) {
                $query .= " AND $table1.rank_site = '$site'";
            }
            if ( ! empty( $group )) {
                $query .= " AND  $table1.group_id = '$group'";
            }
            if ( $date_method == 1 && ! empty( $year ) && ! empty( $month )) {
                $query .= " AND DATE_FORMAT(  $table1.rank_modified, '%Y' ) = '$year'";
                $query .= " AND DATE_FORMAT(  $table1.rank_modified, '%m' ) = '$month'";
            } else if ( $date_method == 2 && ! empty( $year )) {
                $query .= " AND DATE_FORMAT( $table1. rank_modified, '%Y' ) = '$year'";
            }
            
            $query .= " ORDER BY $table1.rank_progress DESC";
            if ( $count != 0 ) {
                $query .= " LIMIT $count OFFSET 0";
            }
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get went down
         *
         * @param  int    $count
         * @param  string $site
         * @param  int    $group
         * @param  int    $date_method
         * @param  int    $year
         * @param  int    $month
         * @return array
         */
        public function get_went_down( $count = 0, $site = '', $group = 0, $date_method = 1, $year = 0, $month = 0 )
        {
            global $wpdb;
            $table1 = $wpdb->prefix . 'iptrank_rank';
            $table2 = $wpdb->prefix . 'iptrank_rank_group';
            $query  = "
                SELECT * 
                    FROM $table1
                    JOIN $table2 ON $table1.group_id = $table2.group_id
                    WHERE 1=1 AND $table1.rank_progress < 0
            ";
            if ( ! empty( $site )) {
                $query .= " AND $table1.rank_site = '$site'";
            }
            if ( ! empty( $group )) {
                $query .= " AND  $table1.group_id = '$group'";
            }
            if ( $date_method == 1 && ! empty( $year ) && ! empty( $month )) {
                $query .= " AND DATE_FORMAT(  $table1.rank_modified, '%Y' ) = '$year'";
                $query .= " AND DATE_FORMAT(  $table1.rank_modified, '%m' ) = '$month'";
            } else if ( $date_method == 2 && ! empty( $year )) {
                $query .= " AND DATE_FORMAT( $table1. rank_modified, '%Y' ) = '$year'";
            }
            
            $query .= " ORDER BY $table1.rank_progress ASC";
            if ( $count != 0 ) {
                $query .= " LIMIT $count OFFSET 0";
            }
            
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get all current ranks
         *
         * @param string $site
         * @param  int    $group
         * @param  int    $date_method
         * @param  int    $year
         * @param  int    $month
         * @return array
         */
        public function get_current_ranks( $site = '', $group = 0, $date_method = 1, $year = 0, $month = 0 )
        {
            global $wpdb;
            $table1 = $wpdb->prefix . 'iptrank_rank';
            $table2 = $wpdb->prefix . 'iptrank_rank_group';
            $query  = "
                SELECT * 
                    FROM $table1
                    JOIN $table2 ON $table1.group_id = $table2.group_id
                    WHERE 1=1
            ";
            if ( ! empty( $site )) {
                $query .= " AND $table1.rank_site = '$site'";
            }
            if ( ! empty( $group )) {
                $query .= " AND  $table1.group_id = '$group'";
            }
            if ( $date_method == 1 && ! empty( $year ) && ! empty( $month )) {
                $query .= " AND DATE_FORMAT(  $table1.rank_modified, '%Y' ) = '$year'";
                $query .= " AND DATE_FORMAT(  $table1.rank_modified, '%m' ) = '$month'";
            } else if ( $date_method == 2 && ! empty( $year )) {
                $query .= " AND DATE_FORMAT( $table1. rank_modified, '%Y' ) = '$year'";
            }
            $query .= " ORDER BY $table1.rank_value ASC";
            
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get rank record
         *
         * @param int $id
         * @return array
         */
        public function get_record( $id )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank_record';
            $sql   = "
                SELECT * 
                    FROM $table 
                    WHERE reco_id = %d
            ";
            $id      = absint( $id );
            $query   = $wpdb->prepare( $sql, $id );
            $results = $wpdb->get_row( $query );
            return $results;
        }
        
        /**
         * Get record for a rank
         *
         * @param int $rankId
         * @param int $perpage
         * @param int $page
         * @return array
         */
        public function get_rank_records( $rankId, $perpage = 0, $page = 1 )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank_record';
            $sql   = "
                SELECT * 
                    FROM $table 
                    WHERE rank_id = %d
                    ORDER BY rank_value
            ";
            if ( $perpage != 0 ) {
                $offset = $perpage * ($page-1);
                $sql   .= " LIMIT $perpage OFFSET $offset";
            }
            $id      = absint( $rankId );
            $query   = $wpdb->prepare( $sql, $id );
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Get sites
         *
         * @return array
         */
        public function get_sites()
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank';
            $query = "
                SELECT DISTINCT rank_site
                    FROM $table 
                    ORDER BY rank_site
            ";
            $results = $wpdb->get_col( $query );
            return $results;
        }
        
        /**
         * Get year max
         *
         * @return array
         */
        public function get_year_max()
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank_record';
            $query = "
                SELECT DATE_FORMAT( reco_date, '%Y') AS year
                    FROM $table 
                    ORDER BY reco_date DESC
                    LIMIT 1 OFFSET 0
            ";
            $result = $wpdb->get_var( $query );
            return $result;
        }
        
        /**
         * Get year min
         *
         * @return array
         */
        public function get_year_min()
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_rank_record';
            $query = "
                SELECT DATE_FORMAT( reco_date, '%Y') AS year
                    FROM $table 
                    ORDER BY reco_date ASC
                    LIMIT 1 OFFSET 0
            ";
            $result = $wpdb->get_var( $query );
            return $result;
        }
        
        /**
         * Insert rank
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insert_rank( $data, $format = array() )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'iptrank_rank';
            $data   = (array)$data;
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }
        
        /**
         * Update rank
         *
         * @param  int   $id
         * @param  array $data
         * @return void
         */
        public function update_rank( $id, $data )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'iptrank_rank';
            $data   = (array)$data;
            $where  = array(
                'rank_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }
        
        /**
         * Delete rank
         *
         * @param  int   $id
         * @return void
         */
        public function delete_rank( $id )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'iptrank_rank';
            $where  = array(
                'rank_id' => absint( $id )
            );
            
            $result = $wpdb->delete( $table, $where );
            return $result;
        }
        
        /**
         * Insert group
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insert_group( $data, $format = array() )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'iptrank_rank_group';
            $data   = (array)$data;
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }
        
        /**
         * Update group
         *
         * @param  int   $id
         * @param  array $data
         * @return void
         */
        public function update_group( $id, $data )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'iptrank_rank_group';
            $data   = (array)$data;
            $where  = array(
                'group_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }
        
        /**
         * Delete group
         *
         * @param  int   $id
         * @return void
         */
        public function delete_group( $id )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'iptrank_rank_group';
            $where  = array(
                'group_id' => absint( $id )
            );
            
            $result = $wpdb->delete( $table, $where );
            return $result;
        }
        
        /**
         * Create group slug
         *
         * @param  string $name
         * @return void
         */
        public function create_group_slug( $name )
        {
            $slug  = strtolower( $name );
            $slug  = preg_replace( '@[^a-zA-Z0-9\-\_]@', '', $slug );
            if ($slugs = $this->get_groups_byslug( $slug ) )
            {
                $count = count( $slugs );
                $count++;
                $slug .= "-$count";
            }
            return $slug;
        }
        
        /**
         * Insert record
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insert_record( $data, $format = array() )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'iptrank_rank_record';
            $data   = (array)$data;
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }
        
        /**
         * Update record
         *
         * @param  int   $id
         * @param  array $data
         * @return void
         */
        public function update_record( $id, $data )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'iptrank_rank_record';
            $data   = (array)$data;
            $where  = array(
                'record_id' => absint( $id )
            );
            $result = $wpdb->update( $table, $data, $where );
            return $result;
        }
        
        /**
         * Delete record
         *
         * @param  int   $id
         * @return void
         */
        public function delete_record( $id )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'iptrank_rank_record';
            $where  = array(
                'record_id' => absint( $id )
            );
            
            $result = $wpdb->delete( $table, $where );
            return $result;
        }
        
        /**
         * Delete record
         *
         * @return void
         */
        public function get_found_item()
        {
            global $wpdb;
            
            $count = 0;
            $query = "SELECT FOUND_ROWS() as count";
            
            $result = $wpdb->get_row( $query );
            if ( ! empty( $result )) {
                $count = $result->count;
            }
            return $count;
        }
        
        
        /**
         * Get logs
         *
         * @param int $perpage
         * @param int $page
         * @return array
         */
        public function get_logs( $start = '', $end = '', $perpage = 0, $page = 1 )
        {
            global $wpdb;
            $table = $wpdb->prefix . 'iptrank_log';
            $query = "
                SELECT SQL_CALC_FOUND_ROWS DISTINCT * 
                    FROM $table 
                    WHERE 1=1";
            
            if ( ! empty( $start ) && ! empty( $end )) 
            {
                if ( $start == $end ) {
                    $query .= " AND DATE_FORMAT( log_date, '%Y-%m-%d' ) = '$start'";
                } else {
                    $query .= " AND log_date BETWEEN '$start' AND '$end'";
                }
            }
            elseif ( ! empty( $start )) {
                $query .= " AND DATE_FORMAT( log_date, '%Y-%m-%d' ) = '$start'";
            }
            elseif ( ! empty( $end )) {
                $query .= " AND DATE_FORMAT( log_date, '%Y-%m-%d' ) = '$end'";
            }
            $query .= " ORDER BY log_date DESC";
            if ( $perpage != 0 ) {
                $offset = $perpage * ($page-1);
                $query .= " LIMIT $perpage OFFSET $offset";
            }
            $results = $wpdb->get_results( $query );
            return $results;
        }
        
        /**
         * Add log
         *
         * @param  string $action
         * @param  string $desc
         * @return void
         */
        public function add_log( $action, $desc )
        {
            $toinsert = array(
                'log_date'      => date( 'c' ),
                'log_action'    => apply_filters( 'single_post_title', $action ),
                'log_desc'      => $desc
            );
            return $this->insert_log( $toinsert );
        }
        
        /**
         * Insert log
         *
         * @param  array $data
         * @param  array $format
         * @return void
         */
        public function insert_log( $data, $format = array() )
        {
            global $wpdb;
            $table  = $wpdb->prefix . 'iptrank_log';
            $data   = (array)$data;
            $result = $wpdb->insert( $table, $data, $format );
            if ( $result ) {
                return $wpdb->insert_id;
            }
            return false;
        }
    }
}