<?php
/*
Plugin Name: WP EZ Rank Checker - Lite
Plugin URI: 
Description: Plugins are tracking your web rank in google
Version: 1.0
Author: InstaPRoduct
Author URI: http://insta-product.com/
*/

/*
    LICENSE: This source file to be part of a system developed by InstaPRoduct. 
    All copyrights use this file and the code in it owned by InstaPRoduct. 
    It is strictly forbidden to use, copy, modify or distribute this file and 
    the code in it without the permission of the developer.
*/

//Change this value for rebranding plugin
define( 'IPTRANK_TITLE', 'WP EZ Rank Checker - Lite' );
define( 'IPTRANK_FILE', __FILE__ );
define( 'IPTRANK_PATH', plugin_dir_path( __FILE__ ) );
define( 'IPTRANK_BASE_URL', plugin_dir_url( __FILE__ ) );

require_once( IPTRANK_PATH . "includes/setting.php" );
require_once( IPTRANK_PATH . "includes/ajax.php" );
require_once( IPTRANK_PATH . "includes/admin.php" );
require_once( IPTRANK_PATH . "includes/db.php" );
require_once( IPTRANK_PATH . "includes/engine.php" );
require_once( IPTRANK_PATH . "includes/model.php" );
require_once( IPTRANK_PATH . "includes/auto-update.php" );

/**
 * Main Class
 */
if ( ! class_exists( 'IPTrank' )) {

    class IPTrank
    {
        /**
         * Setup Plugin
         */
        public static function setup()
        {
            $hook = basename( IPTRANK_FILE, '.php' );
            
            //Install Plugin
            register_activation_hook( __FILE__, array( __CLASS__, 'install' ) );
            //Deactivate plugin
            register_deactivation_hook (__FILE__, array( __CLASS__, 'deactivate' ) );
            //Hook cron job
            add_action( 'iptrank_cron_rank_update', array( __CLASS__, 'cron_job' ) );
            
            //Hook for plugin loaded
            add_action( 'plugins_loaded', array( __CLASS__, 'update_db' ) );
            
            //Register admin menu
            add_action( 'admin_menu', array( __CLASS__, 'admin_menu' ) );
            
            //Plugin init
            add_action( 'admin_init', array( __CLASS__, 'admin_init' ) );
            
            //Add admin style & script
            add_action( 'admin_print_styles', array( __CLASS__, 'admin_style' ), 1000 );
            add_action( 'admin_print_scripts-toplevel_page_iptrank', array( __CLASS__, 'admin_script' ), 1000 );
            add_action( 'admin_print_scripts-'.$hook.'_page_iptrank_group', array( __CLASS__, 'admin_script' ), 1000 );
            add_action( 'admin_print_scripts-'.$hook.'_page_iptrank_report', array( __CLASS__, 'admin_script' ), 1000 );
            add_action( 'admin_print_scripts-'.$hook.'_page_iptrank_research', array( __CLASS__, 'admin_script' ), 1000 );
            add_action( 'admin_print_scripts-'.$hook.'_page_iptrank_setting', array( __CLASS__, 'admin_script' ), 1000 );
            add_action( 'admin_print_scripts-'.$hook.'_page_iptrank_log', array( __CLASS__, 'admin_script' ), 1000 );
        }
        
        /**
         * Install plugin
         *
         * @return void
         */
        public static function install()
        {
            //Db
            $db = new IPTrank_Db();
            $db->install();
            
            //Cron add
            wp_schedule_event( time(), 'daily', 'iptrank_cron_rank_update' );
        }
        
        /**
         * Deactivate plugin
         *
         * @return void
         */
        public static function deactivate()
        {
            //Cron clear
            wp_clear_scheduled_hook( 'iptrank_cron_rank_update' ); 
        }
        
        /**
         * Cron job
         *
         * @return void
         */
        public static function cron_job()
        {
            $is_use_cron = get_option( 'iptrank_cron_job' );
            if ( ! empty( $is_use_cron )) {
                $engine = new IPTrank_Auto_Update();
                $engine->exec();
            }
        }
        
        /**
         * Update Db
         *
         * @return void
         */
        public static function update_db()
        {
            $db = new IPTrank_Db();
            $db->update();
        }
        
        /**
         * Add administrator menu
         *
         * @return void
         */
        public static function admin_menu()
        {
            add_menu_page( 
                IPTRANK_TITLE, 
                IPTRANK_TITLE, 
                'iptrank_manage', 
                'iptrank',
                array( __CLASS__, 'admin_dashboard' ), 
                plugin_dir_url( __FILE__ ) . 'img/menu_icon.png'
            );
            add_submenu_page( 
                'iptrank',
                __( 'Dashboard', 'iptrank' ), 
                __( 'Dashboard', 'iptrank' ), 
                'iptrank_manage', 
                'iptrank',
                array( __CLASS__, 'admin_dashboard' )
            );
            add_submenu_page( 
                'iptrank',
                __( 'Group', 'iptrank' ), 
                __( 'Group', 'iptrank' ), 
                'iptrank_manage', 
                'iptrank_group',
                array( __CLASS__, 'admin_group' )
            );
            add_submenu_page( 
                'iptrank',
                __( 'Reports', 'iptrank' ), 
                __( 'Reports', 'iptrank' ), 
                'iptrank_manage', 
                'iptrank_report',
                array( __CLASS__, 'admin_report' )
            );
            add_submenu_page( 
                'iptrank',
                __( 'Research', 'iptrank' ), 
                __( 'Research', 'iptrank' ), 
                'iptrank_manage', 
                'iptrank_research',
                array( __CLASS__, 'admin_research' )
            );
            add_submenu_page( 
                'iptrank',
                __( 'Logs', 'iptrank' ), 
                __( 'Logs', 'iptrank' ), 
                'iptrank_manage', 
                'iptrank_log',
                array( __CLASS__, 'admin_log' )
            );
            add_submenu_page( 
                'iptrank',
                __( 'Settings', 'iptrank' ), 
                __( 'Settings', 'iptrank' ), 
                'iptrank_manage', 
                'iptrank_setting',
                array( __CLASS__, 'admin_setting' )
            );
        }
        
        /**
         * Admin Init
         *
         * @return void
         */
        public static function admin_init()
        {
            global $wp_roles;
            
            /** 
             * Register Roles
             */
            // Add capability for administrator
            $role = get_role( 'administrator' );
            $role->add_cap( 'iptrank_manage' );
            
            $wp_roles->reinit();
            
            /**
             * Ajax handler
             **/
            $ajax = new IPTrank_Ajax();
            $ajax->exec();
            
            /**
             * Auto update rank
             */
            $engine = new IPTrank_Auto_Update();
            $engine->exec();
        }
        
        /**
         * Display for admin dashboard page
         *
         * @return void
         */
        public static function admin_dashboard()
        {
            $display = new IPTrank_Admin();
            if ( ! empty( $_GET['action'] ) && $_GET['action'] == 'view' ) {
                $display->view();
            } elseif ( ! empty( $_GET['action'] ) && $_GET['action'] == 'view_site' ) {
                $display->view_site();
            } else {
                $display->dashboard();
            }
        }
        
        /**
         * Display for admin group
         *
         * @return void
         */
        public static function admin_group()
        {
            $display = new IPTrank_Admin();
            $display->group();
        }
        
        /**
         * Display for admin report
         *
         * @return void
         */
        public static function admin_report()
        {
            $display = new IPTrank_Admin();
            $display->report();
        }
        
        /**
         * Display for admin research
         *
         * @return void
         */
        public static function admin_research()
        {
            $display = new IPTrank_Admin();
            $display->research();
        }
        
        /**
         * Display for admin setting
         *
         * @return void
         */
        public static function admin_setting()
        {
            $display = new IPTrank_Admin();
            $display->setting();
        }
        
        /**
         * Display for admin log
         *
         * @return void
         */
        public static function admin_log()
        {
            $display = new IPTrank_Admin();
            $display->log();
        }

        /**
         * Print admin style
         *
         * @return void
         */
        public static function admin_style()
        {
            wp_deregister_style('font-awesome-410');
            wp_register_style('font-awesome-410', plugin_dir_url( __FILE__ ) . 'css/font-awesome-4.1.0/css/font-awesome.css', false, '4.0.1');
            wp_enqueue_style('font-awesome-410');
            
            wp_deregister_style('insta-admin-style');
            wp_register_style('insta-admin-style', plugin_dir_url( __FILE__ ) . 'css/ipadmin.css', false, null);
            wp_enqueue_style('insta-admin-style');
            
            wp_deregister_style('insta-ui-style');
            wp_register_style('insta-ui-style', plugin_dir_url( __FILE__ ) . 'css/ui.css', false, null);
            wp_enqueue_style('insta-ui-style');
            
            wp_deregister_style('iptrank-admin-style');
            wp_register_style('iptrank-admin-style', plugin_dir_url( __FILE__ ) . 'css/admin.css', false, null);
            wp_enqueue_style('iptrank-admin-style');
        }

        /**
         * Print Admin Script
         *
         * @return void
         */
        public static function admin_script($hook)
        {
        ?>
            <script type="text/javascript">
                iptrank_base = '<?php echo IPTRANK_BASE_URL; ?>';
                ip_ui_message = 
                {
                    delete_confirm_title: '<?php _e( 'Delete Confirmation', 'iptrank' ); ?>',
                    delete_confirm_info: '<?php _e( 'Are you sure doing this action?', 'iptrank' ); ?>'
                };
            </script>
        <?php
            wp_deregister_script('insta-admin-script');
            wp_enqueue_script(
                'insta-admin-script',
                plugin_dir_url( __FILE__ ) . 'js/ipadmin.js',
                array( 'jquery' ),
                1,
                true
            );
            wp_deregister_script('insta-ui-script');
            wp_enqueue_script(
                'insta-ui-script',
                plugin_dir_url( __FILE__ ) . 'js/ui.js',
                array( 'jquery' ),
                1,
                true
            );
            wp_deregister_script('insta-library-script');
            wp_enqueue_script(
                'insta-library-script',
                plugin_dir_url( __FILE__ ) . 'js/library.js',
                array( 'jquery' ),
                1,
                true
            );
            wp_deregister_script('iptrank-admin-script');
            wp_enqueue_script(
                'iptrank-admin-script',
                plugin_dir_url( __FILE__ ) . 'js/admin.js',
                array( 'jquery' ),
                1,
                true
            );
        }
    }

}
//Run plugin
IPTrank::setup();
?>