<?php
/**
 * Class for handle ajax request
 */
if ( ! class_exists( 'IPTrank_Ajax' )) {

    class IPTrank_Ajax
    {
        /**
         * Take ajax action name
         *
         * @var string
         */
        protected $_action;
        
        /**
         * Take ajax action handlers
         *
         * @var array
         */
        protected $_handlers = array(
            'iptrank_save_keyword'          => 'save_keyword',
            'iptrank_delete_keyword'        => 'delete_keyword',
            'iptrank_delete_keywords'       => 'delete_keywords',
            'iptrank_update_rank'           => 'update_rank',
            'iptrank_update_ranks'          => 'update_ranks',
            'iptrank_save_group'            => 'save_group',
            'iptrank_delete_group'          => 'delete_group',
            'iptrank_save_setting'          => 'save_setting',
            
            'iptrank_rank_report'           => 'rank_report',
            'iptrank_research'              => 'research',
            'iptrank_research_input'        => 'research_input',
            'iptrank_research_list'         => 'research_list',
            'iptrank_research_input_list'   => 'research_input_list'
        );
        
        /**
         * Class constructor
         *
         * @return void
         */
        public function __construct()
        {
            $this->model = new IPTrank_Model();
        }
        
        /**
         * Get action name
         *
         * @return string
         */
        public function getAction()
        {
            return $this->_action;
        }
        
        /**
         * Set action
         *
         * @param  string $action
         * @return IPShortcode_Ajax
         */
        public function setAction( $action )
        {
            $this->_action = sanitize_title( $action );
        }
        
        /**
         * Execute request
         *
         * @param  string $action
         * @return IPShortcode_Ajax
         */
        public function exec()
        {
            $data = array_merge( $_GET, $_POST );
            if ( empty( $data['doajax'] ) || empty( $data['iptrank'] ) || empty( $data['action'] ) ) {
                return true;
            }
            $this->setAction( $data['action'] );
            
            $action   = $this->getAction();
            $handlers = (array)$this->_handlers;
            if ( array_key_exists( $action, $handlers )) {
                $handler = $handlers[$action];
                if ( method_exists( $this, $handler )) {
                    $this->$handler();
                }
            }
            return true;
        }
        
        /**
         * Add/update keyword
         *
         * @return void
         */
        public function save_keyword()
        {
            if ( empty( $_POST['site'] ) || empty( $_POST['keywords'] ) || 
                 empty( $_POST['group'] )
            ) {
                echo json_encode( array(
                    'status'     => 0,
                    'message'    => __( 'Error input data', 'iptrank' )
                ) );
                die();
            }
            $site  = sanitize_url( $_POST['site'] );
            $skey  = trim( $_POST['keywords'] );
            $keys  = explode( "\n", $skey );
            
            if ( ! empty( $_POST['newgroup'] )) {
                $group = $this->_add_new_group( $_POST['newgroup'] );
            } else {
                $group = absint( $_POST['group'] );
            }
            //Insert keywords
            $ids = array();
            if ( ! empty( $keys )) {
                foreach( $keys as $key ){
                    $key = apply_filters( 'single_post_title', $key );
                    //Save log
                    $act = __( 'Add new keyword', 'iptrank' );
                    $msg = sprintf( __( 'Try to add keyword %s for %s.', 'iptrank' ), $key, $site );
                    $this->model->add_log( $act, $msg );
                    
                    if ( $result_id = $this->_insert_keyword( $key, $site, $group )) {
                        $ids[] = $result_id;
                        //Save log
                        $msg = sprintf( __( 'Successfully add keyword #%s.', 'iptrank' ), $result_id );
                        $this->model->add_log( $act, $msg );
                    } else {
                        //Save log
                        $msg = sprintf( __( 'Failed to add keyword #%s.', 'iptrank' ), $result_id );
                        $this->model->add_log( $act, $msg );
                    }
                }
            }
            $content = $this->_get_row_table_ranks( $ids );
            
            $redirect = admin_url() . 'admin.php?page=iptrank&create=1';
            echo json_encode( array(
                'status'     => 1,
                'message'    => __( 'Successfully add keywords', 'iptrank' ),
                'keys'       => $keys,
                'ids'        => $ids,
                'content'    => $content,
                //'redirect'   => $redirect
            ) );
            die();
        }
        
        /**
         * Get row ranks table
         *
         * @return void
         */
        protected function _get_row_table_ranks( $ids )
        {
            //Build table row in buffer
            ob_start();
            
            require_once( IPTRANK_PATH . "includes/row-rank.php" );
            $table = new IPTrank_RowRank( $ids );
            $table->prepare_items();
            $table->display_rows();
            
            //Get content table
            $result = ob_get_contents();
            //End buffer
            ob_end_clean();
            
            return $result;
        }
        
        /**
         * Processing ajax for delete keyword
         *
         * @return void
         */
        public function delete_keyword()
        {
            $item = false;
            if ( ! empty( $_REQUEST['id'] )) {
                $id   = absint( $_REQUEST['id'] );
                $item = $this->model->get_rank( $id );
            }
            if ( ! $item ) {
                echo json_encode( array(
                    'status'   => 0,
                    'message'  => __( 'Error system', 'iptrank' )
                ) );
                die();
            }
            //Save log
            $act = __( 'Delete keyword', 'iptrank' );
            $msg = sprintf( __( 'Try to delete keyword #%s.', 'iptrank' ), $id );
            $this->model->add_log( $act, $msg );
            
            $result = $this->model->delete_rank( $item->rank_id );
            if ( $result ) 
            {
                //Save log
                $msg = sprintf( __( 'Successfully delete keyword #%s.', 'iptrank' ), $id );
                $this->model->add_log( $act, $msg );
                
                echo json_encode( array(
                    'status'   => 1,
                    'message'  => __( 'Successfully remove the keyword', 'iptrank' ),
                    'redirect' => admin_url() . 'admin.php?page=iptrank&delete=1'
                ) );
            } 
            else 
            {
                //Save log
                $msg = sprintf( __( 'Failed to delete keyword #%s.', 'iptrank' ), $id );
                $this->model->add_log( $act, $msg );
                
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed to remove the keyword', 'iptrank' )
                ) );
            }
            die();
        }
        
        /**
         * Processing ajax for delete keywords
         *
         * @return void
         */
        public function delete_keywords()
        {
            $act = __( 'Delete keyword', 'iptrank' );
            if ( ! empty( $_REQUEST['ids'] )) {
                $_id = trim( $_REQUEST['ids'], ',' );
                $ids = explode( ',', $_id );
                foreach( $ids as $id )
                {
                    //Save log
                    $msg = sprintf( __( 'Try to delete keyword #%s.', 'iptrank' ), $id );
                    $this->model->add_log( $act, $msg );
                    
                    $id   = absint( $id );
                    $item = $this->model->get_rank( $id );
                    if ( $item ) {
                        $result = $this->model->delete_rank( $item->rank_id );
                        if ( $result ) {
                            //Save log
                            $msg = sprintf( __( 'Successfully delete keyword #%s.', 'iptrank' ), $id );
                            $this->model->add_log( $act, $msg );
                        } else {
                            //Save log
                            $msg = sprintf( __( 'Failed to delete keyword #%s.', 'iptrank' ), $id );
                            $this->model->add_log( $act, $msg );
                        }
                    }
                }
            }
            echo json_encode( array(
                'status'   => 1,
                'message'  => __( 'Successfully remove the keywords', 'iptrank' )
            ) );
            die();
        }
        
        /**
         * Processing ajax for update ranking
         *
         * @return void
         */
        public function update_rank()
        {
            $act  = __( 'Rank update', 'iptrank' );
            $item = false;
            if ( ! empty( $_REQUEST['id'] )) {
                $id   = absint( $_REQUEST['id'] );
                $item = $this->model->get_rank( $id );
            }
            if ( ! $item ) {
                echo json_encode( array(
                    'status'   => 0,
                    'message'  => __( 'Error system', 'iptrank' )
                ) );
                die();
            }
            //Save log
            $msg = sprintf( __( 'Try to updating rank for keyword #%s.', 'iptrank' ), $id );
            $this->model->add_log( $act, $msg );
            
            $engine = new IPTrank_Engine( $item->rank_keyword, $item->rank_site );
            $rank   = $engine->run();
            
            //Save update data
            $progress = 0;
            if ( $rank > 0 && $rank < 101 ) {
                $progress = $rank - (int)$item->rank_value;
            }
            $date     = date( 'c' );
            $toupdate = array(
                'rank_modified' => $date,
                'rank_value'    => $rank,
                'rank_progress' => $progress,
                'rank_type'     => 1
            );
            $result = $this->model->update_rank( $item->rank_id, $toupdate );
            
            //Insert record
            $toinsert = array(
                'rank_id'       => $item->rank_id,
                'reco_date'     => $date,
                'reco_value'    => $rank,
                'reco_progress' => $progress,
                'reco_data'     => $engine->get_result_data()
            );
            $result = $this->model->insert_record( $toinsert );
            
            if ( $result ) 
            {
                //Save log
                $msg = sprintf( __( 'Successfully updating rank for keyword #%s.', 'iptrank' ), $id );
                $this->model->add_log( $act, $msg );
                
                echo json_encode( array(
                    'status'   => 1,
                    'rank'     => $this->_get_html_rank( $rank, $progress ),
                    'date'     => mysql2date( 'd M y, h:i', $date ),
                    'message'  => __( 'Successfully update ranking', 'iptrank' )
                ) );
            } 
            else 
            {
                //Save log
                $msg = sprintf( __( 'Failed to updating rank for keyword #%s.', 'iptrank' ), $id );
                $this->model->add_log( $act, $msg );
                
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed to update ranking', 'iptrank' )
                ) );
            }
            die();
        }
        
        /**
         * Processing ajax for update ranking
         *
         * @return void
         */
        public function update_ranks()
        {
            $act    = __( 'Rank update', 'iptrank' );
            $result = array(
                'status'   => 1,
                'message'  => __( 'Successfully update ranking', 'iptrank' ),
                'data'     => array()
            );
            if ( ! empty( $_REQUEST['ids'] )) 
            {
                $_id = trim( $_REQUEST['ids'], ',' );
                $ids = explode( ',', $_id );
                foreach( $ids as $id )
                {
                    $id   = absint( $id );
                    $item = $this->model->get_rank( $id );
                    if ( $item ) 
                    {
                        //Save log
                        $msg = sprintf( __( 'Try to updating rank for keyword #%s.', 'iptrank' ), $id );
                        $this->model->add_log( $act, $msg );
            
                        $engine = new IPTrank_Engine( $item->rank_keyword, $item->rank_site );
                        $rank   = $engine->run();
                        
                        //Save update data
                        $progress = 0;
                        if ( $rank > 0 && $rank < 101 ) {
                            $progress = $rank - (int)$item->rank_value;
                        }
                        $date     = date( 'c' );
                        $toupdate = array(
                            'rank_modified' => $date,
                            'rank_value'    => $rank,
                            'rank_progress' => $progress,
                            'rank_type'     => 1
                        );
                        $r = $this->model->update_rank( $item->rank_id, $toupdate );
                        
                        //Insert record
                        $toinsert = array(
                            'rank_id'       => $item->rank_id,
                            'reco_date'     => $date,
                            'reco_value'    => $rank,
                            'reco_progress' => $progress,
                            'reco_data'     => $engine->get_result_data()
                        );
                        $r = $this->model->insert_record( $toinsert );
                        if ( $r ) 
                        {
                            //Save log
                            $msg = sprintf( __( 'Successfully updating rank for keyword #%s.', 'iptrank' ), $item->rank_id );
                            $this->model->add_log( $act, $msg );
                
                            $result['data'][$id] = array(
                                'rank' => $this->_get_html_rank( $rank, $progress ),
                                'date' => mysql2date( 'd M y, h:i', $date )
                            );
                        }
                        else 
                        {
                            //Save log
                            $msg = sprintf( __( 'Failed to updating rank for keyword #%s.', 'iptrank' ), $item->rank_id );
                            $this->model->add_log( $act, $msg );
                        }
                    }
                
                }
            }
            echo json_encode( $result );
            die();
        }
        
        /**
         * Get html rank
         *
         * @return void
         */
        protected function _get_html_rank( $rank, $progress )
        {
            if ( $rank > 100 ) {
                $rank = '100+';
            }
            $rank = '<span class="iptrank-rank-value">' . $rank . '</span>';
            //Progress
            if ( $progress > 0 ) {
                $icon = '<span class="ipadm-btn ipadm-btn-success ipadm-btn-small iptrank-rank-status"><i class="faip fa-long-arrow-up"></i></span>';
            } elseif ( $progress == 0 ) {
                $icon = '<span class="ipadm-btn ipadm-btn-warning ipadm-btn-small iptrank-rank-status"><i class="faip fa-arrows-h"></i></span>';
            } else {
                $icon = '<span class="ipadm-btn ipadm-btn-danger ipadm-btn-small iptrank-rank-status"><i class="faip fa-long-arrow-down"></i></span>';
            }
            $rank .= $icon;
            if ( ! empty( $progress )) {
                if ( $progress > 0 ) {
                    $val = '+'.$progress;
                } else {
                    $val = $progress;
                }
                $rank .= "( $val )";
            }
            return $rank;
        }
        
        /**
         * Processing ajax for get research result
         *
         * @return void
         */
        public function research()
        {
            $model = new IPTrank_Model();
            if ( ! empty( $_POST['keyword'] )) {
                require_once( IPTRANK_PATH . "includes/suggestion.php" );
                $engine = new IPTrank_Suggestion( $_POST['keyword'] );
                $result = $engine->run();
                if ( ! empty( $result ) ) 
                {
                    echo '<ul>';
                    foreach( $result as $kwd ) 
                    {
                        echo '<li>' .
                                '<label>' .
                                    '<input type="checkbox" class="check-all" name="result_check[]" value="' . $kwd . '">' .
                                    '<span>' . $kwd . '</span>' .
                                '</label>' .
                             '</li>';
                    }
                    echo '</ul>';
                }
            }
            die();
        }
        
        /**
         * Processing ajax for get research result
         *
         * @return void
         */
        public function research_input()
        {
            $model = new IPTrank_Model();
            if ( ! empty( $_POST['keyword'] )) {
                require_once( IPTRANK_PATH . "includes/suggestion.php" );
                $engine = new IPTrank_Suggestion( $_POST['keyword'] );
                $result = $engine->run_single();
                if ( ! empty( $result ) ) 
                {
                    echo '<ul>';
                    foreach( $result as $kwd ) 
                    {
                        echo '<li>' .
                                '<label>' .
                                    '<input type="checkbox" class="check-all" name="result_check[]" value="' . $kwd . '">' .
                                    '<span>' . $kwd . '</span>' .
                                '</label>' .
                             '</li>';
                    }
                    echo '</ul>';
                }
            }
            die();
        }
        
        /**
         * Processing ajax for get research result
         *
         * @return void
         */
        public function research_list()
        {
            $model = new IPTrank_Model();
            if ( ! empty( $_POST['keyword'] )) {
                require_once( IPTRANK_PATH . "includes/suggestion.php" );
                $engine = new IPTrank_Suggestion( $_POST['keyword'] );
                $result = $engine->run();
                if ( ! empty( $result ) ) 
                {
                    foreach( $result as $kwd ) 
                    {
                        echo $kwd . "\n";
                    }
                }
            }
            die();
        }
        
        /**
         * Processing ajax for get research result
         *
         * @return void
         */
        public function research_input_list()
        {
            $model = new IPTrank_Model();
            if ( ! empty( $_POST['keyword'] )) {
                require_once( IPTRANK_PATH . "includes/suggestion.php" );
                $engine = new IPTrank_Suggestion( $_POST['keyword'] );
                $result = $engine->run_single();
                if ( ! empty( $result ) ) 
                {
                    foreach( $result as $kwd ) 
                    {
                        echo $kwd . "\n";
                    }
                }
            }
            die();
        }
        
        /**
         * Processing ajax for get content for rank report
         *
         * @return void
         */
        public function rank_report()
        {
            $model = new IPTrank_Model();
            if ( ! empty( $_POST['rankid'] )) :
                $rankid = absint( $_POST['rankid'] );
                if ( $rank = $model->get_rank( $rankid ) ) :
?>
<div class="iptrank-page-subtitle"><h4>
    <?php
    printf( __( 'Ranking Reports of %s for %s', 'iptrank' ), $rank->rank_site, $rank->rank_keyword );
    ?>
</h4></div>
<form action="" method="post">
    <input type="hidden" name="tab_selected" id="iptrank-tab-selected" value="iptrank-tab-chart" />
    <div class="iptrank-tab-nav">
        <ul>
            <li class="iptrank-tab-menu">
                <a href="#iptrank-tab-chart">
                    <i class="faip fa-signal"></i>
                    <span><?php _e( 'Rank Chart', 'iptrank' ); ?></span>
                </a>
            </li>
            <li class="iptrank-tab-menu">
                <a href="#iptrank-tab-record">
                    <i class="faip fa-file-text-o"></i>
                    <span><?php _e( 'Rank Records', 'iptrank' ); ?></span>
                </a>
            </li>
        </ul>
    </div>
    <div class="iptrank-tab-sections">
        <div id="iptrank-tab-chart" class="iptrank-tab-section">
            <div class="iptrank-tab-header">
                <h4><?php _e( 'Rank Chart', 'iptrank' ); ?></h4>
                <h5><?php _e( 'See keyword ranking report in the form of charts.', 'iptrank' ); ?></h5>
            </div>
            <div class="iptrank-fields">
                <script type="text/javascript">
                    var dataTable = new google.visualization.DataTable();
                    dataTable.addColumn( 'date', '<?php _e( 'Date', 'iptrank' ); ?>' );
                    dataTable.addColumn( 'number', '<?php _e( 'Rank', 'iptrank' ); ?>' );
                    
                    var rows = [
                        <?php
                        $results = $model->get_keyword_stat_data( $rank->rank_id );
                        if ( ! empty( $results )) {
                            foreach( $results as $data )
                            {
                                $timestamp = strtotime( $data->rdate );
                                $phpdate   = getdate($timestamp);
                                $value     = (int)$data->rank;
                                
                                echo '[new Date(' 
                                        . $phpdate['year'] . ', '
                                        . ( $phpdate['mon']  - 1 ) . ', '
                                        . $phpdate['mday'] . ',0,0,0), '
                                        . $value
                                    . '],';
                            }
                        }
                        ?>
                    ];
                    dataTable.addRows( rows );
                    
                    var options = {
                        series: [{pointSize:6}]
                    };
                    var chart = new google.visualization.LineChart(document.getElementById('iptrank-chart-div'));
                    chart.draw(dataTable, options);
                </script>
                <div id="iptrank-chart-div" style="width: 100%; height: 400px;"></div>
            </div>
        </div>
        <div id="iptrank-tab-record" class="iptrank-tab-section" style="display:none;">
            <div class="iptrank-tab-header">
                <h4><?php _e( 'Rank Records', 'iptrank' ); ?></h4>
                <h5><?php _e( 'See all keyword ranking report.', 'iptrank' ); ?></h5>
            </div>
            <div class="iptrank-fields">
            
                <div class="ipadm-clear">
                    <div class="iptrank-col-stat">
                        <h4 class="iptrank-page-section"><?php _e( '10 Last Ranking Records' ); ?></h4>
                        <table class="iptrank-stat-table">
                            <thead>
                                <tr>
                                    <th><?php _e( 'No', 'iptrank' ); ?></th>
                                    <th><?php _e( 'Date', 'iptrank' ); ?></th>
                                    <th><?php _e( 'Rank', 'iptrank' ); ?></th>
                                </tr>
                            </thead>
                            <tbody>
                            <?php
                            $records = $model->get_records( $rank->rank_id, 10 );
                            if ( ! empty( $records )) :
                                $idx = 1;
                                foreach( $records as $record ):
                            ?>
                                <tr>
                                    <td><?php echo $idx; ?></td>
                                    <td><?php echo mysql2date( 'd M y, h:i', $record->reco_date ); ?></td>
                                    <td>
                                        <?php
                                        $value = $record->reco_value;
                                        if ( $value == 0 || $value > 100 ) {
                                            $value = '100+';
                                        }
                                        $value = '<span class="iptrank-rank-value">' . $value . '</span>';
                                        //Progress
                                        if ( $record->reco_progress > 0 ) {
                                            $icon = '<span class="ipadm-btn ipadm-btn-success ipadm-btn-small iptrank-rank-status"><i class="faip fa-long-arrow-up"></i></span>';
                                        } elseif ( $record->reco_progress == 0 ) {
                                            $icon = '<span class="ipadm-btn ipadm-btn-warning ipadm-btn-small iptrank-rank-status"><i class="faip fa-arrows-h"></i></span>';
                                        } else {
                                            $icon = '<span class="ipadm-btn ipadm-btn-danger ipadm-btn-small iptrank-rank-status"><i class="faip fa-long-arrow-down"></i></span>';
                                        }
                                        $value .= $icon;
                                        if ( ! empty( $record->reco_progress )) {
                                            $value .= "( {$record->reco_progress} )";
                                        }
                                        echo $value;
                                        ?>
                                    </td>
                                </tr>
                            <?php
                                    $idx++;
                                endforeach;
                            endif;
                            ?>
                            </tbody>
                        </table>
                    </div>
                    <div class="iptrank-col-stat iptrank-col-right">
                        <h4 class="iptrank-page-section"><?php _e( '10 Best Ranking Records' ); ?></h4>
                        <table class="iptrank-stat-table">
                            <thead>
                                <tr>
                                    <th><?php _e( 'No', 'iptrank' ); ?></th>
                                    <th><?php _e( 'Date', 'iptrank' ); ?></th>
                                    <th><?php _e( 'Rank', 'iptrank' ); ?></th>
                                </tr>
                            </thead>
                            <tbody>
                            <?php
                            $records = $model->get_best_records( $rank->rank_id, 10 );
                            if ( ! empty( $records )) :
                                $idx = 1;
                                foreach( $records as $record ):
                            ?>
                                <tr>
                                    <td><?php echo $idx; ?></td>
                                    <td><?php echo mysql2date( 'd M y, h:i', $record->reco_date ); ?></td>
                                    <td>
                                        <?php
                                        $value = $record->reco_value;
                                        if ( $value == 0 || $value > 100 ) {
                                            $value = '100+';
                                        }
                                        $value = '<span class="iptrank-rank-value">' . $value . '</span>';
                                        //Progress
                                        if ( $record->reco_progress > 0 ) {
                                            $icon = '<span class="ipadm-btn ipadm-btn-success ipadm-btn-small iptrank-rank-status"><i class="faip fa-long-arrow-up"></i></span>';
                                        } elseif ( $record->reco_progress == 0 ) {
                                            $icon = '<span class="ipadm-btn ipadm-btn-warning ipadm-btn-small iptrank-rank-status"><i class="faip fa-arrows-h"></i></span>';
                                        } else {
                                            $icon = '<span class="ipadm-btn ipadm-btn-danger ipadm-btn-small iptrank-rank-status"><i class="faip fa-long-arrow-down"></i></span>';
                                        }
                                        $value .= $icon;
                                        if ( ! empty( $record->reco_progress )) {
                                            $value .= "( {$record->reco_progress} )";
                                        }
                                        echo $value;
                                        ?>
                                    </td>
                                </tr>
                            <?php
                                    $idx++;
                                endforeach;
                            endif;
                            ?>
                            </tbody>
                        </table>
                    </div>
                </div>
            </div>
        </div>
    </div>
</form>
<script>
(function($){
    $(document).ready(function()
    {
        $( '.iptrank-manage-tab' ).iptrank_tabMenu();
    });
})(jQuery);
</script>
<?php
                endif;
            endif;
            die();
        }
        
        /**
         * Add new keyword
         *
         * @param  string $key 
         * @param  string $site
         * @param  int    $group
         * @return int|false
         */
        protected function _insert_keyword( $key, $site, $group )
        {
            global $user_ID;
            
            $result = false;
            if ( ! $keyword = $this->model->get_rank_bykey( $key, $site ))
            {
                $engine = new IPTrank_Engine( $key, $site );
                $value  = $engine->run();
                
                if ( empty( $value )) {
                    $value = 0;
                }
                $toinsert = array(
                    'user_id'       => $user_ID,
                    'group_id'      => $group,
                    'rank_created'  => date( 'c' ),
                    'rank_modified' => date( 'c' ),
                    'rank_keyword'  => $key,
                    'rank_site'     => $site,
                    'rank_value'    => $value,
                    'rank_progress' => 0,
                    'rank_type'     => 1,
                    'rank_status'   => 1
                );
                $result = $this->model->insert_rank( $toinsert );
                if ( $result ) 
                {
                    //Insert record
                    $toinsert = array(
                        'rank_id'       => $result,
                        'reco_date'     => date('c'),
                        'reco_value'    => $value,
                        'reco_data'     => $engine->get_result_data()
                    );
                    $this->model->insert_record( $toinsert );
                }
            }
            return $result;
        }
        
        /**
         * Save setting
         *
         * @return void
         */
        public function save_setting()
        {
            if ( ! empty( $_POST['google_country'] )) {
                $value = urlencode( esc_url_raw( $_POST['google_country'] ));
                update_option( 'iptrank_google_country', $value );
            }
            if ( ! empty( $_POST['research_google'] )) {
                $value = urlencode( esc_url_raw( $_POST['research_google'] ));
                update_option( 'iptrank_research_google', $value );
            }
            if ( isset( $_POST['auto_update'] )) {
                $value = absint( $_POST['auto_update'] );
                update_option( 'iptrank_auto_update', $value );
            }
            if ( isset( $_POST['research_letter'] )) {
                $value = apply_filters( 'single_post_title', $_POST['research_letter'] );
                update_option( 'iptrank_research_letter', $value );
            }
            if ( isset( $_POST['cron_job'] )) {
                $value = absint( $_POST['cron_job'] );
                update_option( 'iptrank_cron_job', $value );
            } else {
                update_option( 'iptrank_cron_job', 0 );
            }
        
            $result = array(
                'status'   => 1,
                'message'  => __( 'Successfully save settings.', 'iptrank' )
            );
            echo json_encode( $result );
            die();
        
        }
        
        /**
         * Add/update group
         *
         * @return void
         */
        public function save_group()
        {
            $redirect = false;
            $isAdd    = false;
            $error    = true;
            if ( ! empty( $_POST['isadd'] )) {
                $isAdd = true;
            }
            if ( ! empty( $_POST['group_id'] )) {
                $group_id = absint( $_POST['group_id'] );
                if ( $group = $this->model->get_group( $group_id ) ) {
                    $this->_update_group( $group );
                } else {
                    $group_id = 0;
                }
            } else {
                $group_id = $this->_add_group();
            }
            if ( $group_id ) {
                $error = false;
            }
            if ( $error ) {
                $status    = 0;
                $group_id  = 0;
                if ( $isAdd ) {
                    $message = __( 'Failed to create a group.', 'iptrank' );
                } else {
                    $message = __( 'Failed to save group.', 'iptrank' );
                }
            } else {
                $status   = 1;
                if ( $isAdd ) {
                    $message  = __( 'Successfully create a group.', 'iptrank' );
                    $redirect = admin_url() . 'admin.php?page=iptrank_group&create=1';
                } else {
                    $message  = __( 'Successfully to save group.', 'iptrank' );
                }
            }
            $result = array(
                'status'   => $status,
                'message'  => $message,
                'groupid'  => $group_id
            );
            if ( $redirect ) {
                $result['redirect'] = $redirect;
            }
            echo json_encode( $result );
            die();
        }
        
        /**
         * Add new group
         *
         * @return int
         */
        protected function _add_group()
        {
            global $user_ID;
            
            $info = '';
            if ( ! empty( $_POST['description'] )) {
                $info = $_POST['description'];
            }
            if ( ! empty( $_POST['slug'] )) {
                $slug = apply_filters( 'pre_term_name', $_POST['slug'] );
            } else {
                $slug = $this->model->create_group_slug( $_POST['name'] );
            }
            $data = array(
                'group_created' => date( 'c' ), 
                'group_name'    => apply_filters( 'single_post_title', $_POST['name'] ), 
                'group_slug'    => $slug, 
                'group_info'    => $info, 
                'group_status'  => 1
            );
            $result = $this->model->insert_group( $data );
            return $result;
        }
        
        /**
         * Add new group
         *
         * @param  string $name
         * @return int
         */
        protected function _add_new_group( $name )
        {
            $name = apply_filters( 'single_post_title', $name );
            $slug = $this->model->create_group_slug( $name );
            
            $data = array(
                'group_created' => date( 'c' ), 
                'group_name'    => $name, 
                'group_slug'    => $slug, 
                'group_info'    => '', 
                'group_status'  => 1
            );
            $result = $this->model->insert_group( $data );
            return $result;
        }
        
        /**
         * Update group
         *
         * @param  object $group
         * @return int
         */
        protected function _update_group( $group )
        {
            global $user_ID;
            
            $data = array();
            if ( ! empty( $_POST['name'] )) {
                $data['group_name'] = apply_filters( 'single_post_title', $_POST['name'] );
            }
            if ( ! empty( $_POST['slug'] )) {
                $data['group_slug'] = apply_filters( 'pre_term_name', $_POST['slug'] );
            }
            if ( ! empty( $_POST['description'] )) {
                $data['group_info'] = $_POST['description'];
            }
            $result = $this->model->update_group( $group->group_id, $data );
            return $result;
        }
        
        /**
         * Processing ajax for delete group
         *
         * @return void
         */
        public function delete_group()
        {
            $item = false;
            if ( ! empty( $_REQUEST['id'] )) {
                $id   = absint( $_REQUEST['id'] );
                $item = $this->model->get_group( $id );
            }
            if ( ! $item ) {
                echo json_encode( array(
                    'status'   => 0,
                    'message'  => __( 'Error system', 'iptrank' )
                ) );
                die();
            }
            $result = $this->model->delete_group( $item->group_id );
            if ( $result ) {
                echo json_encode( array(
                    'status'   => 1,
                    'message'  => __( 'Successfully remove the group', 'iptrank' ),
                    'redirect' => admin_url() . 'admin.php?page=iptrank_group&delete=1'
                ) );
            } else {
                echo json_encode( array(
                    'status'  => 0,
                    'message' => __( 'Failed to remove the group', 'iptrank' )
                ) );
            }
            die();
        }
    }
}