<?php
if ( ! class_exists( 'IPTrank_Auto_Update' )) {

    /**
     * Class engine for get rank search engine
     */
    class IPTrank_Auto_Update
    {
        /**
         * Class constructor
         *
         * @return void
         */
        public function __construct(){}
        
        /**
         * Run
         *
         * @return int
         */
        public function exec()
        {
            $time = get_option( 'iptrank_auto_update' );
            if ( empty( $time )) {
                return true;
            }
            $time = absint( $time );
            if ( $this->_need_update( $time ) ){
                return $this->_update();
            }
            return true;
        }
        
        /**
         * Is need update
         *
         * @param  int $every
         * @return boolean
         */
        protected function _need_update( $every )
        {
            $toupdate = false;
            $every    = 24 * 60 * 60 * $every;
            $last     = get_option( 'iptrank_last_auto_update' );
            if ( empty( $last )) {
                $toupdate = true;
            } else 
            {
                $date = mysql2date( 'U', $last );
                $now  = time();
                $next = $date+$every;
                if ( $now > $next ) {
                    $toupdate = true;
                }
            }
            return $toupdate;
        }
        
        /**
         * Update rank
         *
         * @return boolean
         */
        protected function _update()
        {
            $act    = __( 'Rank update', 'iptrank' );
            $result = true;
            $model  = new IPTrank_Model();
            if ( $ranks = $model->get_ranks() )
            {
                //Save log
                $msg = __( 'Start Automatic updating all ranks.', 'iptrank' );
                $model->add_log( $act, $msg );
                
                foreach( $ranks as $item )
                {
                    //Save log
                    $msg = sprintf( __( 'Try to updating rank for keyword #%s.', 'iptrank' ), $item->rank_id );
                    $model->add_log( $act, $msg );
                    
                    $engine = new IPTrank_Engine( $item->rank_keyword, $item->rank_site );
                    $rank   = $engine->run();
                    
                    //Save update data
                    $progress = 0;
                    if ( $rank > 0 && $rank < 101 ) {
                        $progress = $rank - (int)$item->rank_value;
                    }
                    $date     = date( 'c' );
                    $toupdate = array(
                        'rank_modified' => $date,
                        'rank_value'    => $rank,
                        'rank_progress' => $progress,
                        'rank_type'     => 2
                    );
                    $r = $model->update_rank( $item->rank_id, $toupdate );
                    
                    //Insert record
                    $toinsert = array(
                        'rank_id'       => $item->rank_id,
                        'reco_date'     => $date,
                        'reco_value'    => $rank,
                        'reco_progress' => $progress,
                        'reco_data'     => $engine->get_result_data()
                    );
                    $r = $model->insert_record( $toinsert );
                    if ( $r ) {
                        //Save log
                        $msg = sprintf( __( 'Successfully updating rank for keyword #%s.', 'iptrank' ), $item->rank_id );
                        $model->add_log( $act, $msg );
                    } else {
                        //Save log
                        $msg = sprintf( __( 'Failed to updating rank for keyword #%s.', 'iptrank' ), $item->rank_id );
                        $model->add_log( $act, $msg );
                    }
                }
                //Save last update date
                update_option( 'iptrank_last_auto_update', date( 'c' ) );
                
                //Save log
                $msg = __( 'Finish Automatic updating all ranks.', 'iptrank' );
                $model->add_log( $act, $msg );
            }
            return $result;
        }
    }

}
?>