<?php
if ( ! class_exists( 'IPTrank_Db' )) {

    /**
     * Class for manage plugin database
     */
    class IPTrank_Db
    {
        /**
         * Take db version
         *
         * @var string
         */
        public $version = '1.0';
        
        /**
         * Class construct
         *
         * @return void
         */
        public function __construct()
        {
            global $iptrank_db_version;
            $iptrank_db_version = $this->version;
        }
        
        /**
         * Database query
         *
         * @return string
         */
        public function query()
        {
            $sql  = '';
            $sql .= $this->queryRank();
            $sql .= $this->queryRankGroup();
            $sql .= $this->queryRankRecord();
            $sql .= $this->queryLog();
            return $sql;
        }
        
        /**
         * Database query rank
         *
         * @return string
         */
        public function queryRank()
        {
            global $wpdb;
            
            $table_name = $wpdb->prefix . 'iptrank_rank';
            $sql = "CREATE TABLE $table_name (
                        rank_id          int(11)             NOT NULL AUTO_INCREMENT,
                        group_id         int(11)             NOT NULL,
                        user_id          int(11)             NOT NULL,
                        rank_created     datetime  DEFAULT '0000-00-00 00:00:00' NOT NULL,
                        rank_modified    datetime  DEFAULT '0000-00-00 00:00:00' NOT NULL,
                        rank_keyword     varchar(244)       NOT NULL,
                        rank_site        varchar(244)       NOT NULL,
                        rank_value       smallint           NOT NULL,
                        rank_progress    tinyint            NOT NULL,
                        rank_type        tinyint            NOT NULL,
                        rank_status      tinyint            NOT NULL,
                    PRIMARY KEY rank_id (rank_id),
                    KEY group_id (group_id),
                    KEY user_id (user_id)
            );";

            return $sql;
        }
        
        /**
         * Database query rank group
         *
         * @return string
         */
        public function queryRankGroup()
        {
            global $wpdb;
            
            $table_name = $wpdb->prefix . 'iptrank_rank_group';
            $sql = "CREATE TABLE $table_name (
                        group_id         int(11)             NOT NULL AUTO_INCREMENT,
                        group_created    datetime  DEFAULT '0000-00-00 00:00:00' NOT NULL,
                        group_name       varchar(244)       NOT NULL,
                        group_slug       varchar(244)       NOT NULL,
                        group_info       text               NOT NULL,
                        group_status     tinyint            NOT NULL,
                    PRIMARY KEY group_id (group_id)
            );";

            return $sql;
        }
        
        /**
         * Database query rank record
         *
         * @return string
         */
        public function queryRankRecord()
        {
            global $wpdb;
            
            $table_name = $wpdb->prefix . 'iptrank_rank_record';
            $sql = "CREATE TABLE $table_name (
                        reco_id          int(11)             NOT NULL AUTO_INCREMENT,
                        rank_id          int(11)             NOT NULL,
                        reco_date        datetime  DEFAULT '0000-00-00 00:00:00' NOT NULL,
                        reco_value       smallint           NOT NULL,
                        reco_progress    tinyint            NOT NULL,
                        reco_data        text               NOT NULL,
                    PRIMARY KEY reco_id (reco_id),
                    KEY rank_id (rank_id)
            );";

            return $sql;
        }
        
        /**
         * Database query log
         *
         * @return string
         */
        public function queryLog()
        {
            global $wpdb;
            
            $table_name = $wpdb->prefix . 'iptrank_log';
            $sql = "CREATE TABLE $table_name (
                        log_id           int(11)             NOT NULL AUTO_INCREMENT,
                        log_date         datetime  DEFAULT  '0000-00-00 00:00:00' NOT NULL,
                        log_action       varchar(244)        NOT NULL,
                        log_desc         text                NOT NULL,
                    PRIMARY KEY log_id (log_id)
            );";

            return $sql;
        }
        
        /**
         * install
         *
         * @return void
         */
        public function install()
        {
            global $wpdb, $iptrank_db_version;
            $installed_ver = get_option( "iptrank_db_version" );
            
            if( ! $installed_ver || ( $installed_ver != $iptrank_db_version ) ) {

                $sql = $this->query();

                require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
                dbDelta( $sql );

                update_option( "iptrank_db_version", $iptrank_db_version );
            }
        }
        
        /**
         * update
         *
         * @return void
         */
        public function update()
        {
            global $iptrank_db_version;
            if ( get_site_option( 'iptrank_db_version' ) != $iptrank_db_version ) {
                $this->install();
            }
        }
    }

}
?>