<?php
if ( ! class_exists( 'IPTrank_Suggestion' )) {
    
    /**
     * Class engine for get suggestion search
     */
    class IPTrank_Suggestion
    {
        /**
         * Take keyword
         * @var string
         */
        protected $_keyword;
        
        /**
         * Class constructor
         *
         * @param string $keyword
         * @param string $site
         * @return void
         */
        public function __construct( $keyword )
        {
            $this->set_keyword( $keyword );
        }
        
        /**
         * Get keyword
         *
         * @return string
         */
        public function get_keyword()
        {
            return $this->_keyword;
        }
        
        /**
         * Set keyword
         *
         * @param  string $keyword
         * @return object
         */
        public function set_keyword( $keyword )
        {
            $this->_keyword = apply_filters( 'single_post_title', $keyword );
            return $this;
        }
        
        /**
         * Get search engine host url
         *
         * @return string
         */
        public function get_search_host()
        {
            $host   = $this->get_google_host();
            $client = 'firefox';
            $lang   = 'en-US';
            $url    = $host . '/s?';
            $url   .= 'output=search&client=' . $client;
            $url   .= '&hl=' . $lang;
            return $url;
        }
        
        /**
         * Get search engine host url
         *
         * @return string
         */
        public function get_google_host()
        {
            $host = 'http://www.google.com';
            $opt  = get_option( 'iptrank_research_google' );
            if ( ! empty( $opt )) {
                $host = urldecode( $opt );
            }
            $host = trim( $host, '/' );
            return $host;
        }
        
        /**
         * Run
         *
         * @return array
         */
        public function run()
        {
            $keyword = $this->get_keyword();
            $keyword = urlencode( $keyword );
            $result  = array( $keyword );
            
            //Get suggestion for origin keyword
            $data   = $this->_get_result( $keyword );
            $result = $this->_scraping_result( $data );
            
            //Get suggestion by abjad
            $abjad = $this->_get_abjad();
            foreach( $abjad as $char )
            {
                $kwd   = $keyword . '+' . $char;
                $data  = $this->_get_result( $kwd );
                $_res  = $this->_scraping_result( $data );
                if ( ! empty( $_res )) {
                    $result = array_merge( $result, $_res );
                }
            }
            $result = array_unique( $result );
            sort( $result );
            return $result;
        }
        
        /**
         * Run
         *
         * @return array
         */
        public function run_single()
        {
            $keyword = $this->get_keyword();
            $keyword = urlencode( $keyword );
            $result  = array( $keyword );
            
            //Get suggestion for origin keyword
            $data   = $this->_get_result( $keyword );
            $result = $this->_scraping_result( $data );
            
            $result = array_unique( $result );
            sort( $result );
            return $result;
        }
        
        /**
         * Scraping result
         *
         * @param  string $result
         * @return array
         */
        protected function _scraping_result( $result )
        {
            $keywords = array();
            $data     = json_decode( $result, true );
            if ( $data !== null ) {
                $keywords = $data[1];
            }
            return $keywords;
        }
        
        /**
         * Get suggestion
         *
         * @param  string $keyword
         * @return string
         */
        protected function _get_result( $keyword )
        {
            $se_host = $this->get_search_host();
            $se_url  = $se_host . '&q='.$keyword;
            $result  = @file_get_contents( $se_url );
            return $result;
        }
        
        /**
         * Get abjad
         *
         * @return array
         */
        protected function _get_abjad()
        {
            $default = 'a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z';
            $option  = get_option( 'iptrank_research_letter', $default );
            $option  = trim( $option, ',' );
            $abjad   = explode( ',', $option );
            return $abjad;
        }
    }

}
?>